/*
	
	MPVT.cpp:
	SuperCollider support for Ms Pinky's Vinyl Tracking system.
	
    Copyright (c) 2006-2007 Daniel Jones. All rights reserved.
	http://www.erase.net/
	
	----------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
*/

#include "SC_PlugIn.h"
#include "VinylTrackFramework.h"

static InterfaceTable *ft;

struct MPVelocity : public Unit
{
	void *mpvt;
};
struct MPPosition : public Unit
{
	void *mpvt;
};
struct MPSignalPower : public Unit
{
	void *mpvt;
};
struct MPErrorMetric : public Unit
{
	void *mpvt;
};

 
extern "C"
{
	void load(InterfaceTable *inTable);
	void MPVelocity_next(MPVelocity *unit, int inNumSamples);
	void MPVelocity_Ctor(MPVelocity* unit);
	void MPVelocity_Dtor(MPVelocity* unit);
	void MPPosition_next(MPPosition *unit, int inNumSamples);
	void MPPosition_Ctor(MPPosition* unit);
	void MPPosition_Dtor(MPPosition* unit);
	void MPSignalPower_next(MPSignalPower *unit, int inNumSamples);
	void MPSignalPower_Ctor(MPSignalPower* unit);
	void MPSignalPower_Dtor(MPSignalPower* unit);
	void MPErrorMetric_next(MPErrorMetric *unit, int inNumSamples);
	void MPErrorMetric_Ctor(MPErrorMetric* unit);
	void MPErrorMetric_Dtor(MPErrorMetric* unit);
};

void init_mpvt (Unit *unit, void *mpvt)
{
	// set sensible default values
	float absolute = ZIN0(2);
	float threshold = ZIN0(3);
	float sharpness = ZIN0(4);
	float generation = ZIN0(5);
	
	MPVT_SetAbsoluteMode(mpvt, absolute > 0);
	
	// convert amp to db
	threshold = log10(threshold) * 20.0;
	MPVT_SetSignalPowerThreshold(mpvt, threshold);

	MPVT_SetCutoffSharpness(mpvt, sharpness); 
	
	// API handles generation numbering from 0
	MPVT_SetVinylGeneration(mpvt, (int) generation - 1);
}


void load(InterfaceTable *inTable)
{
	ft = inTable;
	DefineDtorUnit(MPVelocity);
	DefineDtorUnit(MPPosition);
	DefineDtorUnit(MPSignalPower);
	DefineDtorUnit(MPErrorMetric);
}



//--------------------------------------------------
// VELOCITY
//--------------------------------------------------

void MPVelocity_Ctor(MPVelocity *unit)
{
	SETCALC(MPVelocity_next);
	
	// create ms pinky object and set initial values	
	unit->mpvt = MPVT_CreateNew(64, unit->mWorld->mSampleRate);
	init_mpvt(unit, unit->mpvt);
	
	MPVelocity_next(unit, 1);
}

void MPVelocity_Dtor(MPVelocity *unit)
{
	MPVT_Destruct(unit->mpvt);
}

void MPVelocity_next(MPVelocity *unit, int inNumSamples)
{
	float *out = OUT(0);
	float *in_left = IN(0);
	float *in_right = IN(1);
	
	void *mpvt = unit->mpvt;
	
	int numCalcs = (int) (inNumSamples / 32.0) + 1;
	Float64 velocityVals[numCalcs],
			powerVals[numCalcs],
			positionVals[numCalcs];
	long num_measurements = numCalcs;
	
	MPVT_ProcessBuffer(mpvt, in_left, in_right, inNumSamples, velocityVals, powerVals, positionVals, num_measurements);
	
	for (int i = 0; i < inNumSamples; ++i)
	{
		out[i] = velocityVals[(int) (i / 32)];
	}
}

//--------------------------------------------------
// POSITION
//--------------------------------------------------

void MPPosition_Ctor(MPPosition *unit)
{
	SETCALC(MPPosition_next);
	
	unit->mpvt = MPVT_CreateNew(64, unit->mWorld->mSampleRate);
	init_mpvt(unit, unit->mpvt);
	
	MPPosition_next(unit, 1);
}

void MPPosition_Dtor(MPPosition *unit)
{
	MPVT_Destruct(unit->mpvt);
}

void MPPosition_next(MPPosition *unit, int inNumSamples)
{
	float *out = OUT(0);
	float *in_left = IN(0);
	float *in_right = IN(1);
	
	void *mpvt = unit->mpvt;
	
	int numCalcs = (int) (inNumSamples / 32.0) + 1;
	Float64 velocityVals[numCalcs],
			powerVals[numCalcs],
			positionVals[numCalcs];
	long num_measurements = numCalcs;
	
	MPVT_ProcessBuffer(mpvt, in_left, in_right, inNumSamples, velocityVals, powerVals, positionVals, num_measurements);
	
	for (int i = 0; i < inNumSamples; ++i)
	{
		out[i] = positionVals[(int) (i / 32)];
	}
}

//--------------------------------------------------
// SIGNAL POWER
//--------------------------------------------------

void MPSignalPower_Ctor(MPSignalPower *unit)
{
	SETCALC(MPSignalPower_next);
	
	unit->mpvt = MPVT_CreateNew(64, unit->mWorld->mSampleRate);
	init_mpvt(unit, unit->mpvt);
	
	MPSignalPower_next(unit, 1);
}

void MPSignalPower_Dtor(MPSignalPower *unit)
{
	MPVT_Destruct(unit->mpvt);
}

void MPSignalPower_next(MPSignalPower *unit, int inNumSamples)
{
	float *out = OUT(0);
	float *in_left = IN(0);
	float *in_right = IN(1);
	
	void *mpvt = unit->mpvt;
	
	int numCalcs = (int) (inNumSamples / 32.0) + 1;
	Float64 velocityVals[numCalcs],
			powerVals[numCalcs],
			positionVals[numCalcs];
	long num_measurements = numCalcs;
	
	MPVT_ProcessBuffer(mpvt, in_left, in_right, inNumSamples, velocityVals, powerVals, positionVals, num_measurements);
	
	for (int i = 0; i < inNumSamples; ++i)
	{
//		out[i] = 10 * log(powerVals[(int) (i / 32)]);
		// leave as float values (0..1)
		out[i] = powerVals[(int) (i / 32)];
	}
}

//--------------------------------------------------
// ERROR METRIC
//--------------------------------------------------

void MPErrorMetric_Ctor(MPErrorMetric *unit)
{
	SETCALC(MPErrorMetric_next);
	
	unit->mpvt = MPVT_CreateNew(64, unit->mWorld->mSampleRate);
	init_mpvt(unit, unit->mpvt);
	
	MPErrorMetric_next(unit, 1);
}

void MPErrorMetric_Dtor(MPErrorMetric *unit)
{
	MPVT_Destruct(unit->mpvt);
}

void MPErrorMetric_next(MPErrorMetric *unit, int inNumSamples)
{
	float *out = OUT(0);
	float *in_left = IN(0);
	float *in_right = IN(1);
	
	void *mpvt = unit->mpvt;
	int numCalcs = (int) (inNumSamples / 32.0) + 1;
	Float64 velocityVals[numCalcs],
			powerVals[numCalcs],
			positionVals[numCalcs];
	long num_measurements = numCalcs;
	
	MPVT_ProcessBuffer(mpvt, in_left, in_right, inNumSamples, velocityVals, powerVals, positionVals, num_measurements);	
	
	Float64 error = MPVT_Query_ErrorMetric(mpvt);
	
	for (int i = 0; i < inNumSamples; ++i)
	{
		out[i] = (float) error;
	}
}

